/* Application Explorer - a general-purpose application cataloger/indexer
 * Copyright (C) 2006  Aliasgar Lokhandwala <d7@freepgs.com> 
 * http://appexp.sourceforge.net/
 *
 * SetupFileHunter.java: The class that helps in searching for 
 * 	"typical" setup files
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

package app.appexplorer.hunt;

import java.io.File;
import java.util.Hashtable;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.LogManager;
import java.util.logging.Logger;

import app.appexplorer.file.IBrwFile;
import app.appexplorer.file.ListedFile;

/**
 * SetupFileHunter is designed to hunt for files designated as commonly used to
 * "Setup" or "Install" applications. The file to be used as a setupfile is
 * given in the "application descriptor" of an application. However if not given
 * this class searches for the most common types like exe, rpm, zip etc
 * 
 * @version 2
 * @author ali
 */

public final class SetupFileHunter {
    /**
     * Contains the file filters to use when searching.
     */
    private Hashtable m_htShowfiles = null;

    /**
     * Link to the BrwFile which will provide essential info for the search.
     */
    private IBrwFile m_brwFile = null;

    /**
     * Used for logging messages.
     */
    private static Logger m_globalLogger = LogManager.getLogManager()
            .getLogger("global");

    /**
     * Creates a hunter that will hunt for setup files, based on information
     * obtained from the supplied "application descriptor"
     * 
     * @param brwFile -
     *            An "application descriptor", must not be null.
     */
    public SetupFileHunter(IBrwFile brwFile) {
        m_brwFile = brwFile;
    }

    /**
     * Converts the vector of File objects to a vector of ListedFile objects.
     * 
     * @param files -
     *            A vector containing File objects. Should not be null, Should
     *            not contain nulls.
     * @return - A vector containing ListedFile objects. Never null.
     * @see ListedFile
     */
    private Vector convertToListedFiles(Vector files) {
        Vector converted = new Vector();
        for (int i = 0; i < files.size(); i++) {
            converted.add(new ListedFile((File) files.get(i)));
        }
        return converted;
    }

    /**
     * Actually performs the search for setup files defined by the attached
     * "application descriptor"
     * 
     * @return - A Vector containing ListedFile objects. Never null, does not
     *         contain nulls.
     * @see FolderSearch
     */
    public Vector getFiles() {
        //get the filters for files to search
        Vector filters = m_brwFile.getSetupFileFilters();
        if (filters.size() == 0) {
            filters = getDefaultFilters();
        }

        //get the base folder for this application
        String sBaseDir = m_brwFile.getContainingFolder();
        boolean bSearchSubDirs = m_brwFile.searchSubFoldersForSetupFiles();

        //do some useful logging
        m_globalLogger.log(Level.INFO, "Searching for setup files in: "
                + sBaseDir);
        m_globalLogger.log(Level.FINE, "Getting files matching: "
                + filters.toString());

        Vector filesfound = new Vector();
        //loop through the list of filters, performing a search on each one
        for (int i = 0; i < filters.size(); i++) {
            FolderSearch s = new FolderSearch(((String) filters.elementAt(i))
                    .toLowerCase(), sBaseDir);
            s.doSearch(bSearchSubDirs);

            //add all files found to the filesfound vector
            filesfound.addAll(s.getFiles());
        }

        //if no files matching the supplied filters are found, search for all
        // files
        if (filesfound.isEmpty()) {
            m_globalLogger
                    .log(Level.WARNING,
                            "No files matching specified filters were found, will show all files");

            return getAllFiles(sBaseDir, bSearchSubDirs);
        }

        return convertToListedFiles(filesfound);
    }

    /**
     * Searches for all files in the applications basedir, ignoring any filters
     * defined in the "application descriptor" <br>
     * This is called from getFiles() if it fails to find files matching the
     * filters specified.
     * 
     * @param sBaseDir -
     *            The root folder to start searching at.
     * @param bSearchSubDirs -
     *            true to search subfolders of sBaseDir.
     * @return - A Vector containing ListedFile objects. Never null, does not
     *         contain nulls.
     */
    private Vector getAllFiles(String sBaseDir, boolean bSearchSubDirs) {
        Vector filesfound = new Vector();

        FolderSearch s = new FolderSearch("*", sBaseDir);
        s.doSearch(bSearchSubDirs);
        filesfound.addAll(s.getFiles());

        return convertToListedFiles(filesfound);
    }

    /**
     * Returns a standard list of common filters.
     * 
     * @return A Vector containing common filters, never null.
     */
    private Vector getDefaultFilters() {

        m_globalLogger.log(Level.INFO,
                "No file filters found, will use default filter set");

        Vector filters = new Vector();
        filters.add("*.exe");
        filters.add("*.zip");
        filters.add("*.msi");
        filters.add("*.rpm");
        filters.add("*.gz");

        return filters;
    }

}